//
//  DEAbstractDecorator.m
//  UITableViewDecoratorExample
//
//  Created by Valentin Radu on 10/01/2013.
//  Copyright (c) 2013 Valentin Radu. All rights reserved.
//

#import "DEAbstractDecorator.h"

@implementation DEAbstractDecorator{
    id _decoratedObject;
}
@dynamic tableItems;

- (id)initWithDecoratedObject:(id<DEDecoratedViewControllerProtocol>) decObj
{
    self = [super init];
    
    if (self)
    {
        _decoratedObject = decObj;
    }
    return self;
}


//the first thing the forwarding mechanism does
//is calling this method if there is no selector
//found
//we override it and try to send the same message to our
//decorated method
//if that fails too, we have nothing better to do than let
//the system rise the exception by returning self
- (id)forwardingTargetForSelector:(SEL)aSelector
{
    if ([_decoratedObject respondsToSelector:aSelector]) {
        return _decoratedObject;
    }
    
    return self;
}

//objective-c is smart enough to know the
//difference between inheritence and this 'fake
//inheritence' we're trying to do
//methods like respondsToSelector: and isKindOfClass: look only
//at the inheritance hierarchy, never at the forwarding chain
//so we force them to consider it by overriding them
- (BOOL)respondsToSelector:(SEL)aSelector
{
    if ( [super respondsToSelector:aSelector] )
        return YES;
    else {
        
        if ([_decoratedObject respondsToSelector:aSelector]) {
            return YES;
        }
        
    }
    return NO;
}

- (BOOL)isKindOfClass:(Class)aClass
{
    if ([super isKindOfClass:aClass]) {
        return YES;
    }
    else {
        
        if ([_decoratedObject isKindOfClass:aClass]) {
            return YES;
        }
        
    }
    
    return NO;
}


@end
