//
//  DEAppDelegate.m
//  UITableViewDecoratorExample
//
//  Created by Valentin Radu on 10/01/2013.
//  Copyright (c) 2013 Valentin Radu. All rights reserved.
//

#import "DEAppDelegate.h"
#import "DERootViewController.h"
#import "DERedDecorator.h"

@implementation DEAppDelegate{
    DEAbstractDecorator * dec;
}

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    
    //init the controller
    //this is just a basic UIViewController subclass implementing DEDecoratedViewControllerProtocol protocol
    //a nice thing about this method is that you can keep your model-controller relation intact
    //so, even if the controler will not be the table view delegate, it will still hold the model data for us
    DERootViewController * viewController = [[DERootViewController alloc] init];
    //our model data is simply a dictionary for the sake of simplicity
    viewController.tableItems = @[@{@"title": @"Africa"},
                                @{@"title": @"America"},
                                @{@"title": @"Asia"},
                                @{@"title": @"Europe"}];
    
    
    //decorate the text label with a color
    dec = [[DERedDecorator alloc] initWithDecoratedObject:viewController];
    //is as easy as this to change it to another color
    //DEAbstractDecorator * dec = [[DEGrayDecorator alloc] initWithDecoratedObject:viewController];
    
    
    
    //then decorate the table view with a footer
    //notice that we pass the prev decoration as parameter
    //this is really nice because we can have a chain of decorations on the same decorated object
    dec = [[DEFooterDecorator alloc] initWithDecoratedObject:dec];
    
    
    //maybe we want a header too
    dec = [[DEHeaderDecorator alloc] initWithDecoratedObject:dec];
    
    
    
    //create the tableview
    UITableView * tableView = [[UITableView alloc] initWithFrame:self.window.frame style:UITableViewStylePlain];
    //delegate to decorator
    tableView.delegate = dec;
    tableView.dataSource = dec;
    //set other prettiness related things
    tableView.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"bkg.png"]];
    tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    
    //assign the view
    viewController.view = tableView;
    
    self.window.rootViewController = viewController;
    
    self.window.backgroundColor = [UIColor whiteColor];
    [self.window makeKeyAndVisible];
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application
{
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

@end
